==> vilistextum-2.3.2.tar.gz: vilistextum-v2.3.2/src/main.c <==
/*
 * vilistextum v2.3.2 21.05.01 - a html to text converter
 * Copyright (C) 1998-2001 Patric Mller
 * bhaak@gmx.net
 * http://www.diax.ch/users/bhaak/vilistextum/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 *
 *
 * history:
 * v0.1 ??.??.???? : lost due Harddisk-Crash
 * v1.x 26.04.1998 : variable width,
 *                   italic, underline, bold
 *                   HR implemented
 * v2.x 13.06.98   : UL, LI implemented
 *      21.06.98   : modified interface
 *      25.06.98   : fixed problem with paragraph-breaking & SGML entities
 *      27.06.98   : LI fixed, implemented IMAGE
 *      28.06.98   : HR: WIDTH, ALIGN,
 *                   DL, DT, DD,
 *                   PRE, TABLE (sort of),
 *                   EM, STRONG implemented
 *      15.07.98   : $91,$92,$93,$94 --> `,',","
 *      26.07.98   : fixed a bug with
 *                   get_attr(): got confused over empty attributs
 *                               <HR NOSHADE>
 *                   paragraph handling: got screwed up if there were
 *                                       only <p>s, but not </p>s
 *      28.07.98   : added "& " for those who don't know &amp;
 *      18.09.98   : added CODE
 *                   added all &xxx;
 *                   added "&#38;"  '&', "&#62;"  '<', "&#60;"  '>'
 *      02.02.99   : added "&#160;" ' ', "&#169;" '', "&#180;" '`',
 *                         "&#228;" '', "&#252;" '', "&#246;" '',
 *                         "&#196;" '', "&#220;" '', "&#214;" '',
 *                         "&#223;" 'ss',
 *                   added RIGHT, /RIGHT,
 *                   added all "&#xxx;",
 *                   AREA, /AREA, MAP, /MAP,
 *                   !--
 *      27.02.99   : better implementation of BLOCKQUOTE
 *      09.03.99   : fixed bug which read wrong breite
 *
 *      01.09.00   : added support for palm, added &#145-152
 *      30.10.00   : added "&#34;"
 *      10.11.00   : bad bug, Remember Character!=String,
 *                   convert B,I,U to ASCII-Characters,
 *                   ignoring now SCRIPT and SUP
 *      30.01.01   : added all special microsoft ascii char between 127-160,
 *                   no more warnings with wall
 *      14.02.01   : better command parsing (still no getopt)
 *                   added microsoft entities
 *      20.02.01   : unknown &#xxx; will be printed as text
 *                   microsoft_entities broke latin1 regognition (how appropriate)
 *                   errorlevel introduced
 *      20.02.01   : SCRIPT was not implemented
 *                   Default Image text editable
 *                   Comment !-- was buggy, stopped at just a >
 *                   get_attr eats whitespace now
 *      22.02.01   : added --remove-empty-alt
 *                   OL are not numbered but get a "-"
 *      26.02.01   : broken entities (&nbsp) are recognized
 *                   removed a small table bug
 *                   no longer gets confused about something like <</b>
 *      09.03.01   : getopt implemented
 *                   added some xhtml like empty tags <br/> <hr/>,
 *                   whitespace error in get_attr fixed
 *                   html 3.2 spec misunderstood, div needs linebreak
 *      08.04.01   : fixed friss_kommentar
 *      09.04.01   : first public release
 *
 *      19.04.01   : --version
 *      20.04.01   : stupid, error messages should go to stderr
 *      23.04.01   : links are shown as references ala lynx
 *                   notitle added
 *                   something broke gnu getopt_long if used with short options and argument
 *
 * TO DO
 * -----
 * check OL vs UL
 * align_nr stimmt was nicht
 *
 */

#include <unistd.h>
#include <string.h>

#define _GNU_SOURCE
#include <getopt.h>

#include <stdio.h>
#include <stdlib.h>

#include "html.h"
#include "fileio.h"

#include "debug.h"

// commandline options
int palm = 0, // if true, enable pda specific restrictions   
  	convert_tags = 0, // if true, convert some tags (eg B,I) to ASCII characters
	  errorlevel = 1, // 0, no errormessages; 1, missing entities; 2, missing tags
	  microsoft = 1, // if true,  128-159 in ASCII-Zeichen umgewandelt 
	  shrink_lines = 0, // no more than 2 empty lines
	  remove_empty_alt = 0, // dont show [] for <IMG ALT=""> 
  	option_links = 0, // show links at end of output
  	option_title = 1; // true -> show title tag, else dont

char *default_image="Image"; // Defaultstring fr IMG ohne ALT

void help()
{
	printf("Usage: vilistextum [OPTIONS] [inputfile|-] [outputfile|-]\n");
	printf("OPTIONS: [--version] [--help|-h] [--palm|-p] [--nomicrosoft|-m] [--shrink-lines|-s] ");
	printf("[--remove-empty-alt|-r] [--defimage|-i string] [--width|-w number] ");
	printf("[--errorlevel|-e number] [--convert-tags|-c] [--links|-l]\n");
	exit(0);
} // end help

// ------------------------------------------------

void parse_args(int argc, char *argv[])
{
  int c=0;
	char *argument;
	/*  int digit_optind = 0;
	 */

  while (1)
  {
		int option_index = 0;
		static struct option long_options[] =
		{
			{"palm",  0, 0, 'p'},
			{"nomicrosoft", 0, 0, 'm'},
			{"shrink-lines", 0, 0, 's'},
			{"remove-empty-alt", 0, 0, 'r'},
			{"defimage", 1, 0, 'i'},
			{"width" , 1, 0, 'w'},
			{"convert-tags", 0, 0,'c'},
			{"errorlevel", 1, 0, 'e'},
			{"help", 0, 0, 'h'},
			{"version", 0, 0, 0},
			{"links", 0, 0, 'l'},
			{"notitle", 0, 0, 't'},
			{0, 0, 0, 0}
		};
		
		c = getopt_long (argc, argv, "pmsriwcehlt",
										 long_options, &option_index);

		// get argument
		if (optarg==NULL) { argument = argv[optind]; }
		else { argument = optarg; } 

		//printf("c: %dc optarg: %s optopt: %d argv[option_index]: %s option: %s\n", c, optarg, optopt, argv[optind], argument); 
		if (c == -1) break;

		switch (c)
		{
		case 0:
			//printf ("option %s", long_options[option_index].name);
			//if (optarg)
			//	printf (" with arg %s", optarg);
			//printf ("\n");
			if (strcmp("version", long_options[option_index].name)==0)
			{
				printf("Vilistextum v2.3.1 (03.05.2001)\n");
				exit(0);
			}
			break;
			/*
		case '0':
		case '1':
		case '2':
			if (digit_optind != 0 && digit_optind != this_option_optind)
				printf ("digits occur in two different argv-elements.\n");
			digit_optind = this_option_optind;
			printf ("option %c\n", c);
			break;
			*/
		case 'p':
      // settings for PDAs
      hr_breite = 30;
      breite=DEF_STR_LEN; 
      palm = 1;
			break;
		case 'w': 
			breite = atoi(argument);
			hr_breite = breite;
			if (breite>DEF_STR_LEN) { fprintf(stderr, "max(breite)=%d\n", DEF_STR_LEN); breite=DEF_STR_LEN; }
			break;
		case 'm': microsoft = 0; break;
		case 's': shrink_lines = 1;	break;
		case 'r': remove_empty_alt = 1; break;
		case 'i': default_image = argument; break;
		case 'c': convert_tags = 1; break;
		case 'e': errorlevel = atoi(argument); break;
		case 'h': help(); break;
		case 'l': option_links = 1; break;
		case 't': option_title = 0; break;

		case '?': // unknown option
			help();
			break;
					
		default:
			fprintf(stderr, "?? getopt returned character code 0%o %c??\n", c, c);
		}
	}

	/*                                                        
	if (optind < argc)
  {
		if (argc-optind!=2) { help(); }
		printf ("non-option ARGV-elements: ");
		while (optind < argc)
      printf ("%s ", argv[optind++]);
		printf ("\n");
	}
	*/
}                                

// ------------------------------------------------

int main(int argc, char *argv[])
{
	parse_args(argc, argv);
	
  if (argc < 3) {	help(); }

	open_files(argv[argc-2],argv[argc-1]);

  html();

	quit();
	return(0);
} // end main

==> vilistextum-2.4.1.tar.gz: vilistextum-2.4.1/src/main.c <==
/*
 * Copyright (C) 1998-2001 Patric Mller
 * bhaak@gmx.net
 * http://www.diax.ch/users/bhaak/vilistextum/
 *
 * Released under the GNU GPL - http://www.gnu.org/copyleft/gpl.html
 *
 *
 * history:
 * v0.1 ??.??.???? : lost due Harddisk-Crash
 * v1.x 26.04.1998 : variable width,
 *                   italic, underline, bold
 *                   HR implemented
 * v2.x 13.06.98   : UL, LI implemented
 *      21.06.98   : modified interface
 *      25.06.98   : fixed problem with paragraph-breaking & SGML entities
 *      27.06.98   : LI fixed, implemented IMAGE
 *      28.06.98   : HR: WIDTH, ALIGN,
 *                   DL, DT, DD,
 *                   PRE, TABLE (sort of),
 *                   EM, STRONG implemented
 *      15.07.98   : $91,$92,$93,$94 --> `,',","
 *      26.07.98   : fixed a bug with
 *                   get_attr(): got confused over empty attributs
 *                               <HR NOSHADE>
 *                   paragraph handling: got screwed up if there were
 *                                       only <p>s, but not </p>s
 *      28.07.98   : added "& " for those who don't know &amp;
 *      18.09.98   : added CODE
 *                   added all &xxx;
 *                   added "&#38;"  '&', "&#62;"  '<', "&#60;"  '>'
 *      02.02.99   : added "&#160;" ' ', "&#169;" '', "&#180;" '`',
 *                         "&#228;" '', "&#252;" '', "&#246;" '',
 *                         "&#196;" '', "&#220;" '', "&#214;" '',
 *                         "&#223;" 'ss',
 *                   added RIGHT, /RIGHT,
 *                   added all "&#xxx;",
 *                   AREA, /AREA, MAP, /MAP,
 *                   !--
 *      27.02.99   : better implementation of BLOCKQUOTE
 *      09.03.99   : fixed bug which read wrong breite
 *
 *      01.09.00   : added support for palm, added &#145-152
 *      30.10.00   : added "&#34;"
 *      10.11.00   : bad bug, Remember Character!=String,
 *                   convert B,I,U to ASCII-Characters,
 *                   ignoring now SCRIPT and SUP
 *      30.01.01   : added all special microsoft ascii char between 127-160,
 *                   no more warnings with wall
 *      14.02.01   : better command parsing (still no getopt)
 *                   added microsoft entities
 *      20.02.01   : unknown &#xxx; will be printed as text
 *                   microsoft_entities broke latin1 regognition (how appropriate)
 *                   errorlevel introduced
 *      20.02.01   : SCRIPT was not implemented
 *                   Default Image text editable
 *                   Comment !-- was buggy, stopped at just a >
 *                   get_attr eats whitespace now
 *      22.02.01   : added --remove-empty-alt
 *                   OL are not numbered but get a "-"
 *      26.02.01   : broken entities (&nbsp) are recognized
 *                   removed a small table bug
 *                   no longer gets confused about something like <</b>
 *      09.03.01   : getopt implemented
 *                   added some xhtml like empty tags <br/> <hr/>,
 *                   whitespace error in get_attr fixed
 *                   html 3.2 spec misunderstood, div needs linebreak
 *      08.04.01   : fixed friss_kommentar
 *      09.04.01   : first public release
 *
 *      19.04.01   : --version
 *      20.04.01   : stupid, error messages should go to stderr
 *      23.04.01   : links are shown as references ala lynx
 *                   notitle added
 *                   something broke gnu getopt_long if used with short options and arguments
 *      03.09.01   : added hexadecimal entities
 *      04.09.01   : Fixed --shrink-lines=n
 *
 * TO DO
 * -----
 * check OL vs UL
 * align_nr stimmt was nicht
 * CENTER and table don't get along well
 *
 */

#include <../config.h>

#include <unistd.h>
#include <string.h>

#ifdef HAVE_GETOPT_H
 #define _GNU_SOURCE
 #include <getopt.h>
#else /* DOS Sun et al */
 #include <stdlib.h>
 #include "getopt_l.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "html.h"
#include "fileio.h"

#include "debug.h"

// commandline options
int palm = 0, // if true, enable pda specific restrictions   
  	convert_tags = 0, // if true, convert some tags (eg B,I) to ASCII characters
	  errorlevel = 1, // 0, no errormessages; 1, missing entities; 2, missing tags
	  microsoft = 1, // if true,  128-159 in ASCII-Zeichen umgewandelt 
	  shrink_lines = 0, // no more than 2 empty lines
	  remove_empty_alt = 0, // dont show [] for <IMG ALT=""> 
  	option_links = 0, // show links at end of output
  	option_title = 1, // true -> show title tag, else dont
    sevenbit = 0; // true -> don't touch any character >127 

char *default_image="Image"; // Defaultstring fr IMG ohne ALT

void help()
{
	printf("Usage: vilistextum [OPTIONS] [inputfile|-] [outputfile|-]\n");
	printf("OPTIONS: [--version] [--help|-h] [--palm|-p] [--nomicrosoft|-m] [--shrink-lines|-s [number]] ");
	printf("[--remove-empty-alt|-r] [--defimage|-i string] [--width|-w number] ");
	printf("[--errorlevel|-e number] [--convert-tags|-c] [--links|-l]\n");
	exit(0);
} // end help

// ------------------------------------------------

void parse_args(int argc, char *argv[])
{
  int c=0;
	char *argument;
	/*  int digit_optind = 0;
	 */

  while (1)
  {
		int option_index = 0;
		static struct option long_options[] =
		{
			{"palm",  0, 0, 'p'},
			{"nomicrosoft", 0, 0, 'm'},
			{"shrink-lines", 2, 0, 's'},
			{"remove-empty-alt", 0, 0, 'r'},
			{"defimage", 1, 0, 'i'},
			{"width" , 1, 0, 'w'},
			{"convert-tags", 0, 0,'c'},
			{"errorlevel", 1, 0, 'e'},
			{"help", 0, 0, 'h'},
			{"version", 0, 0, 0},
			{"links", 0, 0, 'l'},
			{"notitle", 0, 0, 't'},
			{"7bit", 0, 0, '7'},
			{0, 0, 0, 0}
		};

#ifdef DEBUG
		int i=0;
		for (i=0; i<argc; i++)
		{
			fprintf(stderr, "%s ", argv[i]);
		}
		printf("\n");
#endif
		
		c = getopt_long (argc, argv, "pmsriwce:hlt",
										 long_options, &option_index);
#ifdef DEBUG
		fprintf(stderr, "c: %d c: %c \n\n", c,c);
#endif
		// get argument
		if (optarg==NULL) { argument = argv[optind]; }
		else { argument = optarg; } 

		//printf("c: %dc optarg: %s optopt: %d argv[option_index]: %s option: %s\n", c, optarg, optopt, argv[optind], argument); 
		if (c == -1) break;

		switch (c)
		{
		case 0:
#ifdef DEBUG
			fprintf (stderr, "option %s long_index %d", long_options[option_index].name, option_index);
			if (optarg)
				fprintf (stderr, " with arg %s", optarg);
			fprintf (stderr, "\n");
#endif
			if (strcmp("version", long_options[option_index].name)==0)
			{
				printf("Vilistextum %s (%s)\n", VERSION, RELEASEDATE);
				exit(0);
			}
			break;
			/*
		case '0':
		case '1':
		case '2':
			if (digit_optind != 0 && digit_optind != this_option_optind)
				printf ("digits occur in two different argv-elements.\n");
			digit_optind = this_option_optind;
			printf ("option %c\n", c);
			break;
			*/
		case 'p':
      // settings for PDAs
      hr_breite = 30;
      breite=DEF_STR_LEN; 
      palm = 1;
			break;
		case 'w': 
			breite = atoi(argument);
			hr_breite = breite;
			if (breite>DEF_STR_LEN) { fprintf(stderr, "max(breite)=%d\n", DEF_STR_LEN); breite=DEF_STR_LEN; }
			break;
		case 'm': microsoft = 0; break;
		case 's': shrink_lines = atoi(argument); if (shrink_lines==0) { shrink_lines = 1; } break;
		case 'r': remove_empty_alt = 1; break;
		case 'i': default_image = argument; break;
		case 'c': convert_tags = 1; break;
		case 'e': errorlevel = atoi(argument); break;
		case 'h': help(); break;
		case 'l': option_links = 1; break;
		case 't': option_title = 0; break;
		case '7': sevenbit = 1; break;

		case '?': // unknown option
			help();
			break;
					
		default:
			fprintf(stderr, "?? getopt returned character code 0%o %c??\n", c, c);
		}
	}

#ifdef DEBUG                                                   
	if (optind < argc)
  {
		if (argc-optind!=2) { help(); }
		fprintf (stderr, "non-option ARGV-elements: ");
		while (optind < argc)
      fprintf (stderr, "%s ", argv[optind++]);
		fprintf (stderr, "\n");
	}
#endif
}                                

// ------------------------------------------------

int main(int argc, char *argv[])
{
	parse_args(argc, argv);
	
  if (argc < 3) {	help(); }

	open_files(argv[argc-2],argv[argc-1]);

  html();

	quit();
	return(0);
} // end main
